/* Bug 11941: Improper assert map->l_init_called in dlclose.
   Copyright (C) 2016-2017 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

/* This simulates an application using the primary DSO which loads the
   plugin DSO.  */
#include <stdio.h>
#include <stdlib.h>

extern void primary (void);

static int
do_test (void)
{
  printf ("INFO: Starting application.\n");
  primary ();
  printf ("INFO: Exiting application.\n");
  return 0;
}

#include <support/test-driver.c>
