/* shm_unlink -- remove a POSIX shared memory object.  Generic POSIX version.
   Copyright (C) 2001-2017 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <unistd.h>

#if ! _POSIX_MAPPED_FILES
#include <rt/shm_unlink.c>

#else

#include <errno.h>
#include <string.h>
#include "shm-directory.h"


/* Remove shared memory object.  */
int
shm_unlink (const char *name)
{
  SHM_GET_NAME (ENOENT, -1, "");

  int result = unlink (shm_name);
  if (result < 0 && errno == EPERM)
    __set_errno (EACCES);
  return result;
}

#endif
